/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map;
import it.tidalwave.util.logging.Logger;
import java.text.FieldPosition;
import java.text.Format;
import java.text.ParsePosition;
import java.text.ParseException;
import org.openide.util.Lookup;
import it.tidalwave.metadata.MetadataItemFormat;
import it.tidalwave.metadata.spi.MetadataItemFormatProvider;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataItemFormatImpl implements MetadataItemFormat
  {
    private static final String CLASS = MetadataItemFormatImpl.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    private final static Map<Class<?>, MetadataItemFormatProvider> formatProviderMapByClass = new HashMap<Class<?>, MetadataItemFormatProvider>();
    
    protected final static Format DEFAULT_FORMAT = new Format() 
      {
        @Override
        @Nonnull 
        public StringBuffer format (@CheckForNull final Object object, 
                                    @Nonnull final StringBuffer toAppendTo, 
                                    @Nonnull final FieldPosition pos) 
          {
            toAppendTo.append((object != null) ? object.toString() : "");
            return toAppendTo;
          }

        @Override
        @Nonnull 
        public Object parseObject (@Nonnull final String source, 
                                   @Nonnull final ParsePosition pos) 
          {
            pos.setIndex(pos.getIndex() + source.length() + 1);
            return source;
          }
      };
    
    public MetadataItemFormatImpl()
      {
        for (final MetadataItemFormatProvider<?> provider : Lookup.getDefault().lookupAll(MetadataItemFormatProvider.class))
          {
            formatProviderMapByClass.put(provider.getItemClass(), provider);
          }
      }
    
    @Nonnull 
    public String getItemDisplayName (@Nonnull final Class<?> itemClass)
      {
        final MetadataItemFormatProvider formatProvider = formatProviderMapByClass.get(itemClass);
        return (formatProvider != null) ? formatProvider.getItemDisplayName() : itemClass.getSimpleName();
      }
    
    @Nonnull 
    public final String format (@Nonnull final Class<?> itemClass, 
                                @Nonnull final String propertyName,
                                @Nonnull final Object value)    
      {
        return findFormat(itemClass, propertyName).format(value);
      }
    
    @Nonnull 
    public final <T> T parse (@Nonnull final Class<?> itemClass, 
                              @Nonnull final String propertyName, 
                              @Nonnull final String value) 
      throws ParseException
      {
        return (T)findFormat(itemClass, propertyName).parseObject(value);
      }
    
    @Nonnull 
    public String getPropertyDisplayName (@Nonnull final Class<?> itemClass, 
                                          @Nonnull final String propertyName)
      {
        final MetadataItemFormatProvider formatProvider = formatProviderMapByClass.get(itemClass);
        return (formatProvider != null) ? formatProvider.getDisplayName(propertyName) : propertyName;
      }

    @Nonnull 
    private Format findFormat (@Nonnull final Class<?> itemClass, 
                               @Nonnull final String propertyName) 
      {
        logger.fine("findFormat(%s, %s", itemClass, propertyName);
        final MetadataItemFormatProvider formatProvider = formatProviderMapByClass.get(itemClass);
        final Format result = (formatProvider != null) ? formatProvider.getFormat(propertyName) : DEFAULT_FORMAT;
        logger.finer(">>>> returning %s", result);
        
        return result;
      }
  }
